//----------------------------------------------
// NFC Packet System with TRT
// 1. Using software shift registers
// to transmit the bits
// 2. Baud rate = 1200 Baud (bit time = 833us)
// 
// Reference: IRPack System from ECE 4760
// 
// Packet start char is '#'
// Packet end char is '%'
// Do not use # or % in a message
// rx_status = 0 means good data
// rx_status = 2 ; // buffer overrun
// rx_status = 1 ; // no data --> timeout
// rx_status = 3 ; // bad transmitter id
// rx_status = 4 ; // bad check sum
//----------------------------------------------

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include <util/delay.h>
#include <string.h>

/* CPU frequency */
#define F_CPU 16000000UL
/* UART1 baud rate */
// for NFC Module
#define NFC_UART_BAUD  19200 // 19.2kbit/s

// ISR
#define SUSPEND cli();
#define RESUME sei();

//************************
//timeout counter  
// time to give up and assume you will never get a response   
#define nfc_rx_timeout 100 //milliseconds
#define nfc_tx_timeout 100

volatile unsigned long tx_send_time, rx_rec_time;

//************************
// string buffers and ISR comm buffers
#define buffer_size 64
volatile char nfc_tx_buffer[buffer_size], nfc_rx_buffer[buffer_size];
// count for filling(sending) buffers
volatile char nfc_tx_count, nfc_rx_count;
// read flags
volatile char nfc_tx_ready, nfc_rx_ready;
// start and end characters for packet
#define start_token '#'
#define end_token '%'

//************************

// NFC send
// Input transmitter id and string packet payload
void nfc_send_packet(char tx_id, char data[]);

// NFC Receive
// input expected transmitter id
// returns zero if payload is valid and returns payload
// 1 means no data; 2 means buffer overrun; 3 means bad tx id; 4 means bad checksum
char nfc_rec_packet(char tx_id, char data[]);

// NFC Packet System Initialization
void nfc_init(void);
